/*
	LWCOLLADA, LightWawe plugin for COLLADA.
	Copyright (C) 2004 Yoshikuni Nomura
	If you want contact to me, please send a e-mail to mail@nomuraz.com .

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <float.h>
#include <math.h>
#include "COLLADAIO.h"

COLLADAIO::COLLADAIO()
:pPaser( NULL ), pDocument( NULL )
{
}

COLLADAIO::~COLLADAIO()
{
	delete pPaser;
}

bool COLLADAIO::Load( std::string FilaName )
{
	pPaser = new xmlpp::DomParser();
	pPaser->parse_file( FilaName.c_str() );
	pDocument = pPaser->get_document();

	return true;
}

// This function is temporary use.
bool COLLADAIO::GetGeometry( std::vector < std::string > &NameVector )
{
	xmlpp::NodeSet GeometryNodeSet = pDocument->get_root_node()->find( "//*[local-name()='geometry']" );
	for( xmlpp::NodeSet::iterator it = GeometryNodeSet.begin(); it != GeometryNodeSet.end(); ++it ){
		NameVector.push_back( GetAttributeValue( *it, "name" ) );
	}

	return true;
}

bool COLLADAIO::GetGeometryPosition( const std::vector < CLXML_POLYGONS >::iterator it_ply, CL_POLYGONS &Polygons )
{
	CLXML_INPUT *pInput = NULL;
	unsigned int InputVectorSize = (unsigned int)it_ply->InputVector.size();
	int InputPos = 0;
	for( unsigned int i = 0; i < InputVectorSize; ++i ){
		if( it_ply->InputVector[i].semantic == "POSITION" ){
			pInput = &it_ply->InputVector[i];
			InputPos = i;
			break;
		}
	}
	if( !pInput ) return false;

	std::vector < std::vector < double > > &PositionVector = Polygons.PositionVector;
	char Buffer[1024];
	sprintf( Buffer, "//*[@name='%s']", pInput->source.c_str() + 1 ); // To remove #, I increment the pointer.
	xmlpp::NodeSet SourceNodeSet = pDocument->get_root_node()->find( Buffer );

	if( SourceNodeSet.size() == 0 ) return false;
	xmlpp::Node *pSourceNode = *SourceNodeSet.begin();

	CLXML_SOURCE Source;
	GetSource( pSourceNode, Source );

	CLXML_ACCESSOR &Accessor = Source.TechniqueVector[0].AccessorVector[0]; // !!! DENGEROUS

	sprintf( Buffer, "//*[@name='%s']", Accessor.source.c_str() + 1 ); // To remove #, I increment the pointer.
	xmlpp::NodeSet ArrayNodeSet = pDocument->get_root_node()->find( Buffer );

	if( ArrayNodeSet.size() == 0 ) return false;
	xmlpp::Node *pArrayNode = *ArrayNodeSet.begin();

	CLXML_ARRAY Array;
	GetArray( pArrayNode, Array );

	for( unsigned long j = 0; j < Accessor.count; ++j ){
		std::vector < double > fv;
		for( unsigned long i = 0; i < Accessor.stride; ++i ){
			fv.push_back( atof( Array.ContentTokenVector[j * Accessor.stride + i].c_str() ) );
		}
		PositionVector.push_back( fv );
	}

	std::vector < std::vector < long > > &IndexVector = Polygons.IndexVector;
	std::vector< CLXML_P > &PVector = it_ply->PVector;

	for( unsigned int k = 0; k < PVector.size() && k < it_ply->count; ++k ){
		std::vector < long > lv;
		for( unsigned int i = 0; i < PVector[k].p.size(); i += InputVectorSize ){
			lv.push_back( PVector[k].p[i + InputPos] );
		}

		// This implementation is very suspicious.
		if( PVector[k].h.size()!= 0 ){

			long HoleStartIndex = lv.back();
			std::vector < double > HoleStartPosition = PositionVector[lv.back()];

			double MinLength = DBL_MAX;
			unsigned int Index = 0;
			for( unsigned int j = 0; j < PVector[k].h.size(); j += InputVectorSize ){
				std::vector < double > HolePosition = PositionVector[PVector[k].h[j + InputPos]];

				double Length = 0;
				for( unsigned int i = 0; i < HoleStartPosition.size(); ++i ){
					Length += pow( HoleStartPosition[i] - HolePosition[i], 2 );
				}

				if( MinLength > Length ){
					MinLength = Length;
					Index = j;
				}
			}

			for( unsigned int i = Index; i < PVector[k].h.size(); i += InputVectorSize ){
				lv.push_back( PVector[k].h[i + InputPos] );
			}

			for( unsigned int i = 0; i < Index; i += InputVectorSize ){
				lv.push_back( PVector[k].h[i + InputPos] );
			}

			lv.push_back( PVector[k].h[Index + InputPos] );
			lv.push_back( HoleStartIndex );
		}

		IndexVector.push_back( lv );
	}

	return true;
}

bool COLLADAIO::GetGeometryTexcoord( const std::vector < CLXML_POLYGONS >::iterator it_ply, CL_POLYGONS &Polygons )
{
	CLXML_INPUT *pInput = NULL;
	unsigned int InputVectorSize = (unsigned int)it_ply->InputVector.size();
	int InputPos = 0;
	for( unsigned int i = 0; i < InputVectorSize; ++i ){
		if( it_ply->InputVector[i].semantic == "TEXCOORD" ){
			pInput = &it_ply->InputVector[i];
			InputPos = i;
			break;
		}
	}
	if( !pInput ) return false;

	std::vector < std::vector < double > > &TexcoordVector = Polygons.TexcoordVector;
	char Buffer[1024];
	sprintf( Buffer, "//*[@name='%s']", pInput->source.c_str() + 1 ); // To remove #, I increment the pointer.
	xmlpp::NodeSet SourceNodeSet = pDocument->get_root_node()->find( Buffer );

	if( SourceNodeSet.size() == 0 ) return false;
	xmlpp::Node *pSourceNode = *SourceNodeSet.begin();

	CLXML_SOURCE Source;
	GetSource( pSourceNode, Source );

	// !!! DENGEROUS
	// I assume that if TechniqueVector has an accessor, that TechniqueVector describes UV coordinates.
	CLXML_ACCESSOR &Accessor = CLXML_ACCESSOR(); // CLXML_ACCESSOR() is dummy
	for( unsigned int i = 0; i < Source.TechniqueVector.size(); ++i ){
		if( Source.TechniqueVector[i].AccessorVector.size() == 1 ){
			Accessor = Source.TechniqueVector[i].AccessorVector[0];
			break;
		}
	}

	sprintf( Buffer, "//*[@name='%s']", Accessor.source.c_str() + 1 ); // To remove #, I increment the pointer.
	xmlpp::NodeSet ArrayNodeSet = pDocument->get_root_node()->find( Buffer );

	if( ArrayNodeSet.size() == 0 ) return false;
	xmlpp::Node *pArrayNode = *ArrayNodeSet.begin();

	CLXML_ARRAY Array;
	GetArray( pArrayNode, Array );

	for( unsigned long j = 0; j < Accessor.count; ++j ){
		std::vector < double > fv;
		for( unsigned long i = 0; i < Accessor.stride; ++i ){
			fv.push_back( atof( Array.ContentTokenVector[j * Accessor.stride + i].c_str() ) );
		}
		TexcoordVector.push_back( fv );
	}

	std::vector < std::vector < long > > &UVIndexVector = Polygons.UVIndexVector;
	std::vector< CLXML_P > &PVector = it_ply->PVector;

	for( unsigned int j = 0; j < PVector.size() && j < it_ply->count; ++j ){
		std::vector < long > lv;
		for( unsigned int i = 0; i < PVector[j].p.size(); i += InputVectorSize ){
			lv.push_back( PVector[j].p[i + InputPos] );
		}

		UVIndexVector.push_back( lv );
	}

	return true;
}

bool COLLADAIO::GetGeometry( std::string Name, CL_GEOMETRY &Geometry )
{
	char Buffer[1024];
	sprintf( Buffer, "//*[local-name()='geometry'][@name='%s']", Name.c_str() );
	xmlpp::NodeSet GeometryNodeSet = pDocument->get_root_node()->find( Buffer );
	if( GeometryNodeSet.size() != 1 ) return false;

	CLXML_GEOMETRY Geo;
	GetGeometry( *GeometryNodeSet.begin(), Geo );

	// <xs:choice minOccurs="0" maxOccurs="1">
	//  <xs:element ref="mesh" />
	//	<xs:element ref="extra">
	// Here, extra node is ignored.

	if( Geo.MeshVector.size() != 1 ) return false;

	CLXML_MESH &Mesh = *Geo.MeshVector.begin();

	// case: polygons
	for( std::vector < CLXML_POLYGONS >::iterator it_ply = Mesh.PolygonsVector.begin(); it_ply != Mesh.PolygonsVector.end(); ++it_ply ){
		Geometry.Mesh.Polygons.push_back( CL_POLYGONS() );

		// case: POSITION
		COLLADAIO::GetGeometryPosition( it_ply, Geometry.Mesh.Polygons.back() );

		// case: TEXCOORD
		// At this time, I ignore like below items.
		/*
              <param name="SCount" type="int" flow="IN">2</param>
              <param name="TCount" type="int" flow="IN">2</param>
              <param name="SOffset" type="float" flow="IN">0.000000</param>
              <param name="TOffset" type="float" flow="IN">0.000000</param>
              <param name="SScaling" type="float" flow="IN">1.000000</param>
              <param name="TScaling" type="float" flow="IN">1.000000</param>
              <param name="RotationX" type="float" flow="IN">0.000000</param>
              <param name="RotationY" type="float" flow="IN">45.000000</param>
              <param name="RotationZ" type="float" flow="IN">0.000000</param>
		*/
		COLLADAIO::GetGeometryTexcoord( it_ply, Geometry.Mesh.Polygons.back() );
	}


	return true;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////

// -- Dataflow Elements --

bool COLLADAIO::GetAccessor( xmlpp::Node *pNode, CLXML_ACCESSOR &Accessor )
{
	Accessor.count  = atoi( GetAttributeValue( pNode, "count" ).c_str() );
	Accessor.offset = atoi( GetAttributeValue( pNode, "offset" ).c_str() );
	Accessor.source = GetAttributeValue( pNode, "source" );
	Accessor.stride = atoi( GetAttributeValue( pNode, "stride" ).c_str() );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "param" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PARAM Param;
		GetParam( *it, Param );
		Accessor.ParamVector.push_back( Param );
	}

	return true;
}

bool COLLADAIO::GetArray( xmlpp::Node *pNode, CLXML_ARRAY &Array )
{
	Array.count = atoi( GetAttributeValue( pNode, "count" ).c_str() );
	Array.name  = GetAttributeValue( pNode, "name" );

	std::string s = GetAttributeValue( pNode, "type" );
	if( s == "int"   ) Array.type = CLXML_ARRAY::ARRAY_INT; 
	if( s == "float" ) Array.type = CLXML_ARRAY::ARRAY_FLOAT; 
	if( s == "Name"  ) Array.type = CLXML_ARRAY::ARRAY_NAME; 
	if( s == "token" ) Array.type = CLXML_ARRAY::ARRAY_TOKEN;

	Array.ContentTokenVector = TokenizeStrings( GetContent( pNode ) );

	return true;
}

bool COLLADAIO::GetInput( xmlpp::Node *pNode, CLXML_INPUT &Input )
{
	Input.id		= GetAttributeValue( pNode, "id" );
	Input.semantic	= GetAttributeValue( pNode, "semantic" );
	Input.source	= GetAttributeValue( pNode, "source" );

	return true;
}

bool COLLADAIO::GetParam( xmlpp::Node *pNode, CLXML_PARAM &Param )
{
	std::string s = GetAttributeValue( pNode, "flow" );
	if( s == "IN"    ) Param.flow = CLXML_PARAM::FLOW_IN;
	if( s == "OUT"   ) Param.flow = CLXML_PARAM::FLOW_OUT;
	if( s == "INOUT" ) Param.flow = CLXML_PARAM::FLOW_INOUT;

	Param.name      = GetAttributeValue( pNode, "name" );
	Param.semantic  = GetAttributeValue( pNode, "semantic" );
	Param.type      = GetAttributeValue( pNode, "type" );

	// This is only contained in 'entry node'.
	Param.qualifier = GetAttributeValue( pNode, "qualifier" );

	Param.ContentTokenVector = TokenizeStrings( GetContent( pNode ) );

	return true;
}

bool COLLADAIO::GetSource( xmlpp::Node *pNode, CLXML_SOURCE &Source )
{
	Source.name = GetAttributeValue( pNode, "name" );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "array" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_ARRAY Array;
		GetArray( *it, Array );
		Source.ArrayVector.push_back( Array );
	}

	NodeList = pNode->get_children( "technique" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_TECHNIQUE_SOURCE Technique;

		Technique.profile = GetAttributeValue( (*it), "profile" );

		xmlpp::Node::NodeList AssetNodeList = (*it)->get_children( "asset" );
		for( xmlpp::Node::NodeList::iterator it_Ast = AssetNodeList.begin(); it_Ast != AssetNodeList.end(); ++it_Ast ){
			CLXML_ASSET Asset;
			GetAsset( *it_Ast, Asset );
			Technique.AssetVector.push_back( Asset );
		}

		xmlpp::Node::NodeList AccessorNodeList = (*it)->get_children( "accessor" );
		for( xmlpp::Node::NodeList::iterator it_Acc = AccessorNodeList.begin(); it_Acc != AccessorNodeList.end(); ++it_Acc ){
			CLXML_ACCESSOR Accessor;
			GetAccessor( *it_Acc, Accessor );
			Technique.AccessorVector.push_back( Accessor );
		}

		xmlpp::Node::NodeList ParamNodeList = (*it)->get_children( "param" );
		for( xmlpp::Node::NodeList::iterator it_param = ParamNodeList.begin(); it_param != ParamNodeList.end(); ++it_param ){
			CLXML_PARAM Param;
			GetParam( *it_param, Param );
			Technique.ParamVector.push_back( Param );
		}

		xmlpp::Node::NodeList ProgramNodeList = (*it)->get_children( "program" );
		for( xmlpp::Node::NodeList::iterator it_prg = ProgramNodeList.begin(); it_prg != ProgramNodeList.end(); ++it_prg ){
			CLXML_PROGRAM Program;
			GetProgram( *it_prg, Program );
			Technique.ProgramVector.push_back( Program );
		}

		Source.TechniqueVector.push_back( Technique );
	}

	return true;
}

// -- Geometry Container Elements --

bool COLLADAIO::GetMesh( xmlpp::Node *pNode, CLXML_MESH &Mesh )
{
	Mesh.name = GetAttributeValue( pNode, "name" );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "source" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_SOURCE Source;
		GetSource( *it, Source );
		Mesh.SourceVector.push_back( Source );
	}

	NodeList = pNode->get_children( "lines" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_LINES Lines;
		GetLines( *it, Lines );
		Mesh.LinesVector.push_back( Lines );
	}

	NodeList = pNode->get_children( "linestrips" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_LINESTRIPS Linestrips;
		GetLinestrips( *it, Linestrips );
		Mesh.LinestripsVector.push_back( Linestrips );
	}

	NodeList = pNode->get_children( "polygons" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_POLYGONS Polygons;
		GetPolygons( *it, Polygons );
		Mesh.PolygonsVector.push_back( Polygons );
	}

	NodeList = pNode->get_children( "triangles" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_TRIANGLES Triangles;
		GetTriangles( *it, Triangles );
		Mesh.TrianglesVector.push_back( Triangles );
	}

	NodeList = pNode->get_children( "trifans" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_TRIFANS Trifans;
		GetTrifans( *it, Trifans );
		Mesh.TrifansVector.push_back( Trifans );
	}

	NodeList = pNode->get_children( "tristrips" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_TRISTRIPS Tristrips;
		GetTristrips( *it, Tristrips );
		Mesh.TristripsVector.push_back( Tristrips );
	}

	return true;
}

// -- Collation Elements --

// This function is not checked. Because there is no sample in examples directly. 
bool COLLADAIO::GetLines( xmlpp::Node *pNode, CLXML_LINES &Lines )
{
	Lines.count    = atoi( GetAttributeValue( pNode, "count" ).c_str() );
	Lines.material = atoi( GetAttributeValue( pNode, "material" ).c_str() );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "input" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_INPUT Input;
		GetInput( *it, Input );
		Lines.InputVector.push_back( Input );
	}

	NodeList = pNode->get_children( "param" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PARAM Param;
		GetParam( *it, Param );
		Lines.ParamVector.push_back( Param );
	}

	NodeList = pNode->get_children( "p" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		std::vector < std::string > Token = TokenizeStrings( GetContent( *it ) );
		CTXML_LISTOFINTS ListOfInts;
		for( unsigned int i = 0; i < Token.size(); ++i ){
			ListOfInts.push_back( atoi( Token[i].c_str() ) );
		}

		Lines.PVector.push_back( ListOfInts );
	}

	return true;
}

// This function is not checked. Because there is no sample in examples directly. 
bool COLLADAIO::GetLinestrips( xmlpp::Node *pNode, CLXML_LINESTRIPS &Linestrips )
{
	Linestrips.count    = atoi( GetAttributeValue( pNode, "count" ).c_str() );
	Linestrips.material = atoi( GetAttributeValue( pNode, "material" ).c_str() );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "input" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_INPUT Input;
		GetInput( *it, Input );
		Linestrips.InputVector.push_back( Input );
	}

	NodeList = pNode->get_children( "param" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PARAM Param;
		GetParam( *it, Param );
		Linestrips.ParamVector.push_back( Param );
	}

	NodeList = pNode->get_children( "p" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		std::vector < std::string > Token = TokenizeStrings( GetContent( *it ) );
		CTXML_LISTOFINTS ListOfInts;
		for( unsigned int i = 0; i < Token.size(); ++i ){
			ListOfInts.push_back( atoi( Token[i].c_str() ) );
		}

		Linestrips.PVector.push_back( ListOfInts );
	}

	return true;
}

bool COLLADAIO::GetPolygons( xmlpp::Node *pNode, CLXML_POLYGONS &Polygons )
{
	Polygons.count    = atoi( GetAttributeValue( pNode, "count" ).c_str() );
	Polygons.material = GetAttributeValue( pNode, "material" ).c_str();

	xmlpp::Node::NodeList NodeList = pNode->get_children( "input" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_INPUT Input;
		GetInput( *it, Input );
		Polygons.InputVector.push_back( Input );
	}

	NodeList = pNode->get_children( "param" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PARAM Param;
		GetParam( *it, Param );
		Polygons.ParamVector.push_back( Param );
	}

	NodeList = pNode->get_children( "p" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		std::vector < std::string > Token = TokenizeStrings( GetContent( *it ) );
		CLXML_P PNode;
		for( unsigned int i = 0; i < Token.size(); ++i ){
			PNode.p.push_back( atoi( Token[i].c_str() ) );
		}

		// CAUTION!!!!!!  This implementation may be unsafe.
		xmlpp::Node::NodeList HNodeList = (*it)->get_children( "h" );
		if( HNodeList.size() ){
			xmlpp::Node::NodeList PNodeChildrenList = (*it)->get_children();
			for( xmlpp::Node::NodeList::iterator it_child = PNodeChildrenList.begin(); it_child != PNodeChildrenList.end(); ++it_child ){
				std::string s = ((xmlpp::ContentNode*)(*it_child))->get_name();
				if( s == "h" ){
					xmlpp::Node::NodeList HNodeChildrenList = (*it_child)->get_children();
					std::vector < std::string > Token;
					if( HNodeChildrenList.size() ){
						Token = TokenizeStrings( 
							((xmlpp::ContentNode*)(*HNodeChildrenList.begin()))->get_content() );
					}
					else{
						Token = TokenizeStrings(
							((xmlpp::ContentNode*)(*(++it_child)))->get_content() );
					}

					for( unsigned int i = 0; i < Token.size(); ++i ){
						PNode.h.push_back( atoi( Token[i].c_str() ) );
					}
				}
			}
		}

		Polygons.PVector.push_back( PNode );
	}

	return true;
}

// This function is not checked. Because there is no sample in examples directly. 
bool COLLADAIO::GetTriangles( xmlpp::Node *pNode, CLXML_TRIANGLES &Triangles )
{
	Triangles.count    = atoi( GetAttributeValue( pNode, "count" ).c_str() );
	Triangles.material = atoi( GetAttributeValue( pNode, "material" ).c_str() );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "input" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_INPUT Input;
		GetInput( *it, Input );
		Triangles.InputVector.push_back( Input );
	}

	NodeList = pNode->get_children( "param" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PARAM Param;
		GetParam( *it, Param );
		Triangles.ParamVector.push_back( Param );
	}

	NodeList = pNode->get_children( "p" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		std::vector < std::string > Token = TokenizeStrings( GetContent( *it ) );
		CTXML_LISTOFINTS ListOfInts;
		for( unsigned int i = 0; i < Token.size(); ++i ){
			ListOfInts.push_back( atoi( Token[i].c_str() ) );
		}

		Triangles.PVector.push_back( ListOfInts );
	}

	return true;
}

// This function is not checked. Because there is no sample in examples directly. 
bool COLLADAIO::GetTrifans( xmlpp::Node *pNode, CLXML_TRIFANS &Trifans )
{
	Trifans.count    = atoi( GetAttributeValue( pNode, "count" ).c_str() );
	Trifans.material = atoi( GetAttributeValue( pNode, "material" ).c_str() );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "input" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_INPUT Input;
		GetInput( *it, Input );
		Trifans.InputVector.push_back( Input );
	}

	NodeList = pNode->get_children( "param" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PARAM Param;
		GetParam( *it, Param );
		Trifans.ParamVector.push_back( Param );
	}

	NodeList = pNode->get_children( "p" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		std::vector < std::string > Token = TokenizeStrings( GetContent( *it ) );
		CTXML_LISTOFINTS ListOfInts;
		for( unsigned int i = 0; i < Token.size(); ++i ){
			ListOfInts.push_back( atoi( Token[i].c_str() ) );
		}

		Trifans.PVector.push_back( ListOfInts );
	}

	return true;
}

// This function is not checked. Because there is no sample in examples directly. 
bool COLLADAIO::GetTristrips( xmlpp::Node *pNode, CLXML_TRISTRIPS &Tristrips )
{
	Tristrips.count    = atoi( GetAttributeValue( pNode, "count" ).c_str() );
	Tristrips.material = atoi( GetAttributeValue( pNode, "material" ).c_str() );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "input" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_INPUT Input;
		GetInput( *it, Input );
		Tristrips.InputVector.push_back( Input );
	}

	NodeList = pNode->get_children( "param" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PARAM Param;
		GetParam( *it, Param );
		Tristrips.ParamVector.push_back( Param );
	}

	NodeList = pNode->get_children( "p" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		std::vector < std::string > Token = TokenizeStrings( GetContent( *it ) );
		CTXML_LISTOFINTS ListOfInts;
		for( unsigned int i = 0; i < Token.size(); ++i ){
			ListOfInts.push_back( atoi( Token[i].c_str() ) );
		}

		Tristrips.PVector.push_back( ListOfInts );
	}

	return true;
}

// -- Transformational Elements --
bool COLLADAIO::GetLookAt( xmlpp::Node *pNode, CLXML_LOOKAT &LookAt )
{
	std::vector < std::string > Token = TokenizeStrings( GetContent( pNode ) );
	CTXML_LISTOFINTS ListOfInts;
	for( unsigned int i = 0; i < Token.size(); ++i ){
		LookAt.ListOfFloats.push_back( atof( Token[i].c_str() ) );
	}

	return true;
}

bool COLLADAIO::GetMatrix( xmlpp::Node *pNode, CLXML_MATRIX &Matrix )
{
	std::vector < std::string > Token = TokenizeStrings( GetContent( pNode ) );
	CTXML_LISTOFINTS ListOfInts;
	for( unsigned int i = 0; i < Token.size(); ++i ){
		Matrix.Matrix4.push_back( atof( Token[i].c_str() ) );
	}

	return true;
}

// This function is not checked. Because there is no sample in examples directly. 
bool COLLADAIO::GetPerspective( xmlpp::Node *pNode, CLXML_PERSPECTIVE &Perspective )
{
	Perspective.Float = atof( GetContent( pNode ).c_str() );

	return true;
}

bool COLLADAIO::GetRotate( xmlpp::Node *pNode, CLXML_ROTATE &Rotate )
{
	std::vector < std::string > Token = TokenizeStrings( GetContent( pNode ) );
	for( unsigned int i = 0; i < Token.size(); ++i ){
		Rotate.Float4.push_back( atof( Token[i].c_str() ) );
	}

	return true;
}

// This function is not checked. Because there is no sample in examples directly. 
bool COLLADAIO::GetScale( xmlpp::Node *pNode, CLXML_SCALE &Scale )
{
	std::vector < std::string > Token = TokenizeStrings( GetContent( pNode ) );
	for( unsigned int i = 0; i < Token.size(); ++i ){
		Scale.Float3.push_back( atof( Token[i].c_str() ) );
	}

	return true;
}

// This function is not checked. Because there is no sample in examples directly. 
bool COLLADAIO::GetSkew( xmlpp::Node *pNode, CLXML_SKEW &Skew )
{
	std::vector < std::string > Token = TokenizeStrings( GetContent( pNode ) );
	for( unsigned int i = 0; i < Token.size(); ++i ){
		Skew.ListOfFloats.push_back( atof( Token[i].c_str() ) );
	}

	return true;
}

bool COLLADAIO::GetTranslate( xmlpp::Node *pNode, CLXML_TRANSLATE &Translate )
{
	std::vector < std::string > Token = TokenizeStrings( GetContent( pNode ) );
	for( unsigned int i = 0; i < Token.size(); ++i ){
		Translate.Float3.push_back( atof( Token[i].c_str() ) );
	}

	return true;
}

// -- Material and Shading Elements --

bool COLLADAIO::GetImage( xmlpp::Node *pNode, CLXML_IMAGE &Image )
{
	Image.name   = GetAttributeValue( pNode, "name" );
	Image.source = GetAttributeValue( pNode, "source" );
	Image.format = GetAttributeValue( pNode, "format" );
	Image.height = atoi( GetAttributeValue( pNode, "height" ).c_str() );
	Image.width  = atoi( GetAttributeValue( pNode, "width" ).c_str() );
	Image.depth  = atoi( GetAttributeValue( pNode, "depth" ).c_str() );

	return true;
}

bool COLLADAIO::GetLight( xmlpp::Node *pNode, CLXML_LIGHT &Light )
{
	Light.name   = GetAttributeValue( pNode, "name" );

	std::string s = GetAttributeValue( pNode, "type" );
	if( s == "AMBIENT"     ) Light.type = CLXML_LIGHT::LIGHT_AMBIENT;
	if( s == "DIRECTIONAL" ) Light.type = CLXML_LIGHT::LIGHT_DIRECTIONAL;
	if( s == "POINT"       ) Light.type = CLXML_LIGHT::LIGHT_POINT;
	if( s == "SPOT"        ) Light.type = CLXML_LIGHT::LIGHT_SPOT;

	xmlpp::Node::NodeList NodeList = pNode->get_children( "asset" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_ASSET Asset;
		GetAsset( *it, Asset );
		Light.AssetVector.push_back( Asset );
	}

	NodeList = pNode->get_children( "param" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PARAM Param;
		GetParam( *it, Param );
		Light.ParamVector.push_back( Param );
	}

	return true;
}

bool COLLADAIO::GetMaterial( xmlpp::Node *pNode, CLXML_MATERIAL &Material )
{
	Material.name = GetAttributeValue( pNode, "name" );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "asset" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_ASSET Asset;
		GetAsset( *it, Asset );
		Material.AssetVector.push_back( Asset );
	}

	NodeList = pNode->get_children( "param" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PARAM Param;
		GetParam( *it, Param );
		Material.ParamVector.push_back( Param );
	}

	NodeList = pNode->get_children( "shader" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_SHADER Shader;
		GetShader( *it, Shader );
		Material.ShaderVector.push_back( Shader );
	}

	return true;
}

bool COLLADAIO::GetPass( xmlpp::Node *pNode, CLXML_PASS &Pass )
{
	xmlpp::Node::NodeList NodeList = pNode->get_children( "param" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PARAM Param;
		GetParam( *it, Param );
		Pass.ParamVector.push_back( Param );
	}

	NodeList = pNode->get_children( "input" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_INPUT Input;
		GetInput( *it, Input );
		Pass.InputVector.push_back( Input );
	}

	NodeList = pNode->get_children( "program" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PROGRAM Program;
		GetProgram( *it, Program );
		Pass.ProgramVector.push_back( Program );
	}

	return true;
}


bool COLLADAIO::GetShader( xmlpp::Node *pNode, CLXML_SHADER &Shader )
{
	Shader.name = GetAttributeValue( pNode, "name" );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "param" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PARAM Param;
		GetParam( *it, Param );
		Shader.ParamVector.push_back( Param );
	}

	NodeList = pNode->get_children( "technique" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_TECHNIQUE_SHADER Technique;

		Technique.profile = GetAttributeValue( (*it), "profile" );

		xmlpp::Node::NodeList AssetNodeList = (*it)->get_children( "asset" );
		for( xmlpp::Node::NodeList::iterator it_ast = AssetNodeList.begin(); it_ast != AssetNodeList.end(); ++it_ast ){
			CLXML_ASSET Asset;
			GetAsset( *it_ast, Asset );
			Technique.AssetVector.push_back( Asset );
		}

		xmlpp::Node::NodeList ParamNodeList = (*it)->get_children( "param" );
		for( xmlpp::Node::NodeList::iterator it_param = ParamNodeList.begin(); it_param != ParamNodeList.end(); ++it_param ){
			CLXML_PARAM Param;
			GetParam( *it_param, Param );
			Technique.ParamVector.push_back( Param );
		}

		xmlpp::Node::NodeList PassNodeList = (*it)->get_children( "pass" );
		for( xmlpp::Node::NodeList::iterator it_pass = PassNodeList.begin(); it_pass != PassNodeList.end(); ++it_pass ){
			CLXML_PASS Pass;
			GetPass( *it_pass, Pass );
			Technique.PassVector.push_back( Pass );
		}

		xmlpp::Node::NodeList ProgramNodeList = (*it)->get_children( "program" );
		for( xmlpp::Node::NodeList::iterator it_prg = ProgramNodeList.begin(); it_prg != ProgramNodeList.end(); ++it_prg ){
			CLXML_PROGRAM Program;
			GetProgram( *it_prg, Program );
			Technique.ProgramVector.push_back( Program );
		}

		Shader.TechniqueVector.push_back( Technique );
	}

	return true;
}

bool COLLADAIO::GetTexture( xmlpp::Node *pNode, CLXML_TEXTURE &Texture )
{
	Texture.name = GetAttributeValue( pNode, "name" );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "param" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PARAM Param;
		GetParam( *it, Param );
		Texture.ParamVector.push_back( Param );
	}

	NodeList = pNode->get_children( "technique" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_TECHNIQUE_TEXTURE Technique;

		Technique.profile = GetAttributeValue( (*it), "profile" );

		xmlpp::Node::NodeList AssetNodeList = (*it)->get_children( "asset" );
		for( xmlpp::Node::NodeList::iterator it_ast = AssetNodeList.begin(); it_ast != AssetNodeList.end(); ++it_ast ){
			CLXML_ASSET Asset;
			GetAsset( *it_ast, Asset );
			Technique.AssetVector.push_back( Asset );
		}

		xmlpp::Node::NodeList InputNodeList = (*it)->get_children( "input" );
		for( xmlpp::Node::NodeList::iterator it_input = InputNodeList.begin(); it_input != InputNodeList.end(); ++it_input ){
			CLXML_INPUT Input;
			GetInput( *it_input, Input );
			Technique.InputVector.push_back( Input );
		}

		xmlpp::Node::NodeList ParamNodeList = (*it)->get_children( "param" );
		for( xmlpp::Node::NodeList::iterator it_param = ParamNodeList.begin(); it_param != ParamNodeList.end(); ++it_param ){
			CLXML_PARAM Param;
			GetParam( *it_param, Param );
			Technique.ParamVector.push_back( Param );
		}

		xmlpp::Node::NodeList ProgramNodeList = (*it)->get_children( "program" );
		for( xmlpp::Node::NodeList::iterator it_prg = ProgramNodeList.begin(); it_prg != ProgramNodeList.end(); ++it_prg ){
			CLXML_PROGRAM Program;
			GetProgram( *it_prg, Program );
			Technique.ProgramVector.push_back( Program );
		}

		Texture.TechniqueVector.push_back( Technique );
	}

	return true;
}

// -- Procedural Elements --

bool COLLADAIO::GetCode( xmlpp::Node *pNode, CLXML_CODE &Code )
{
	Code.profile  = GetAttributeValue( pNode, "profile" );
	Code.semantic = GetAttributeValue( pNode, "semantic" );
	Code.url      = GetAttributeValue( pNode, "url" );

	Code.Content = GetContent( pNode );

	return true;
}

bool COLLADAIO::GetEntry( xmlpp::Node *pNode, CLXML_ENTRY &Entry )
{
	Entry.function = GetAttributeValue( pNode, "function" );
	Entry.semantic = GetAttributeValue( pNode, "semantic" );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "param" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PARAM Param;
		GetParam( *it, Param );
		Entry.ParamVector.push_back( Param );
	}

	return true;
}

bool COLLADAIO::GetProgram( xmlpp::Node *pNode, CLXML_PROGRAM &Program )
{
	Program.name = GetAttributeValue( pNode, "name" );
	Program.url  = GetAttributeValue( pNode, "url" );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "asset" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_ASSET Asset;
		GetAsset( *it, Asset );
		Program.AssetVector.push_back( Asset );
	}

	NodeList = pNode->get_children( "param" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PARAM Param;
		GetParam( *it, Param );
		Program.ParamVector.push_back( Param );
	}

	NodeList = pNode->get_children( "entry" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_ENTRY Entry;
		GetEntry( *it, Entry );
		Program.EntryVector.push_back( Entry );
	}

	NodeList = pNode->get_children( "code" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_CODE Code;
		GetCode( *it, Code );
		Program.CodeVector.push_back( Code );
	}

	return true;
}

//-- Object Elements --
bool COLLADAIO::GetCamera( xmlpp::Node *pNode, CLXML_CAMERA &Camera )
{
	Camera.name = GetAttributeValue( pNode, "name" );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "asset" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_ASSET Asset;
		GetAsset( *it, Asset );
		Camera.AssetVector.push_back( Asset );
	}

	NodeList = pNode->get_children( "technique" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_CAMERA_TECHNIQUE Technique;

		Technique.profile = GetAttributeValue( (*it), "profile" );

		xmlpp::Node::NodeList OpticsNodeList = (*it)->get_children( "optics" );
		for( xmlpp::Node::NodeList::iterator it_opt = OpticsNodeList.begin(); it_opt != OpticsNodeList.end(); ++it_opt ){
			CLXML_OPTICS Optics;
			xmlpp::Node::NodeList ProgramNodeList = (*it_opt)->get_children( "program" );
			for( xmlpp::Node::NodeList::iterator it_prg = ProgramNodeList.begin(); it_prg != ProgramNodeList.end(); ++it_prg ){
				CLXML_PROGRAM Program;
				GetProgram( *it_prg, Program );
				Optics.ProgramVector.push_back( Program );
			}

			Technique.OpticsVector.push_back( Optics );
		}

		xmlpp::Node::NodeList ImagerNodeList = (*it)->get_children( "imager" );
		for( xmlpp::Node::NodeList::iterator it_img = ImagerNodeList.begin(); it_img != ImagerNodeList.end(); ++it_img ){
			CLXML_IMAGER Imager;
			xmlpp::Node::NodeList ProgramNodeList = (*it_img)->get_children( "program" );
			for( xmlpp::Node::NodeList::iterator it_prg = ProgramNodeList.begin(); it_prg != ProgramNodeList.end(); ++it_prg ){
				CLXML_PROGRAM Program;
				GetProgram( *it_prg, Program );
				Imager.ProgramVector.push_back( Program );
			}

			Technique.ImagerVector.push_back( Imager );
		}

		Camera.TechniqueVector.push_back( Technique );
	}

	return true;
}

bool COLLADAIO::GetAsset( xmlpp::Node *pNode, CLXML_ASSET &Asset )
{
	xmlpp::Node::NodeList NodeList = pNode->get_children( "author" );
	if( NodeList.size() == 1 ) Asset.author = (*NodeList.begin())->get_name();

	NodeList = pNode->get_children( "authoring_tool" );
	if( NodeList.size() == 1 ) Asset.authoring_tool = GetContent( *NodeList.begin() );

	NodeList = pNode->get_children( "created" );
	if( NodeList.size() == 1 ) Asset.created = GetContent( *NodeList.begin() );

	NodeList = pNode->get_children( "modified" );
	if( NodeList.size() == 1 ) Asset.modified = GetContent( *NodeList.begin() );

	NodeList = pNode->get_children( "revision" );
	if( NodeList.size() == 1 ) Asset.revision = GetContent( *NodeList.begin() );

	NodeList = pNode->get_children( "source_data" );
	if( NodeList.size() == 1 ) Asset.source_data = GetContent( *NodeList.begin() );

	NodeList = pNode->get_children( "copyright" );
	if( NodeList.size() == 1 ) Asset.copyright = GetContent( *NodeList.begin() );

	NodeList = pNode->get_children( "title" );
	if( NodeList.size() == 1 ) Asset.title = GetContent( *NodeList.begin() );

	NodeList = pNode->get_children( "subject" );
	if( NodeList.size() == 1 ) Asset.subject = GetContent( *NodeList.begin() );

	NodeList = pNode->get_children( "keywords" );
	if( NodeList.size() == 1 ) Asset.keywords = GetContent( *NodeList.begin() );

	NodeList = pNode->get_children( "comments" );
	if( NodeList.size() == 1 ) Asset.comments = GetContent( *NodeList.begin() );

	NodeList = pNode->get_children( "up_axis" );
	if( NodeList.size() == 1 ){
		std::string s = GetContent( *NodeList.begin() );
		if( s == "X_UP" ) Asset.up_axis = CLXML_ASSET::UPAXIS_X_UP;
		if( s == "Y_UP" ) Asset.up_axis = CLXML_ASSET::UPAXIS_Y_UP;
		if( s == "Z_UP" ) Asset.up_axis = CLXML_ASSET::UPAXIS_Z_UP;
	}

	NodeList = pNode->get_children( "unit" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_UNIT Unit;
		Unit.meter = atof( GetAttributeValue( *it, "meter" ).c_str() );
		Unit.name  = GetAttributeValue( *it, "name" );
		Asset.UnitVector.push_back( Unit );
	}

	return true;
}

bool COLLADAIO::GetInstance( xmlpp::Node *pNode, CLXML_INSTANCE &Instance )
{
	Instance.url = GetAttributeValue( pNode, "url" );

	return true;
}

// This function is not checked. Because there is no sample in examples directly and also in specification pdf. 
bool COLLADAIO::GetObject( xmlpp::Node *pNode, CLXML_OBJECT &Object )
{
	Object.name = GetAttributeValue( pNode, "name" );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "modifier" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_MODIFIER Modifier;

		xmlpp::Node::NodeList TechniqueNodeList = (*it)->get_children( "technique" );
		for( xmlpp::Node::NodeList::iterator it_tech = TechniqueNodeList.begin(); it_tech != TechniqueNodeList.end(); ++it_tech ){
			CLXML_OBJECTXML_TECHNIQUE Technique;

			xmlpp::Node::NodeList InputNodeList = (*it_tech)->get_children( "input" );
			for( xmlpp::Node::NodeList::iterator it_inp = InputNodeList.begin(); it_inp != InputNodeList.end(); ++it_inp ){
				CLXML_INPUT Input;
				GetInput( *it_inp, Input );
				Technique.InputVector.push_back( Input );
			}

			xmlpp::Node::NodeList ProgramNodeList = (*it_tech)->get_children( "program" );
			for( xmlpp::Node::NodeList::iterator it_prg = ProgramNodeList.begin(); it_prg != ProgramNodeList.end(); ++it_prg ){
				CLXML_PROGRAM Program;
				GetProgram( *it_prg, Program );
				Technique.ProgramVector.push_back( Program );
			}

			Modifier.TechniqueVector.push_back( Technique );
		}

		Object.ModifierVector.push_back( Modifier );
	}

	return true;
}

// This function is not implemented completely.
// mmm..., I don't know how can I can implement it.
bool COLLADAIO::GetExtra( xmlpp::Node *pNode, CLXML_EXTRA &Extra )
{
	Extra.name = GetAttributeValue( pNode, "name" );

	return true;
}

// -- Hierarchical Elements --

bool COLLADAIO::GetBoundingBox( xmlpp::Node *pNode, CLXML_BOUNDINGBOX &BoundingBox )
{
	xmlpp::Node::NodeList NodeList = pNode->get_children( "min" );
	if( NodeList.size() == 1 ){
		std::vector < std::string > Token = TokenizeStrings( GetContent( *NodeList.begin() ) );
		for( unsigned int i = 0; i < Token.size(); ++i ){
			BoundingBox.Min.push_back( atof( Token[i].c_str() ) );
		}
	}

	NodeList = pNode->get_children( "max" );
	if( NodeList.size() == 1 ){
		std::vector < std::string > Token = TokenizeStrings( GetContent( *NodeList.begin() ) );
		for( unsigned int i = 0; i < Token.size(); ++i ){
			BoundingBox.Max.push_back( atof( Token[i].c_str() ) );
		}
	}

	return true;
}

bool COLLADAIO::GetNode( xmlpp::Node *pNode, CLXML_NODE &Node )
{
	Node.name = GetAttributeValue( pNode, "name" );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "lookat" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_LOOKAT LookAt;
		GetLookAt( *it, LookAt );
		Node.LookAtVector.push_back( LookAt );
	}

	NodeList = pNode->get_children( "matrix" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_MATRIX Matrix;
		GetMatrix( *it, Matrix );
		Node.MatrixVector.push_back( Matrix );
	}

	NodeList = pNode->get_children( "perspective" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PERSPECTIVE Perspective;
		GetPerspective( *it, Perspective );
		Node.PerspectiveVector.push_back( Perspective );
	}

	NodeList = pNode->get_children( "rotate" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_ROTATE Rotate;
		GetRotate( *it, Rotate );
		Node.RotateVector.push_back( Rotate );
	}

	NodeList = pNode->get_children( "scale" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_SCALE Scale;
		GetScale( *it, Scale );
		Node.ScaleVector.push_back( Scale );
	}

	NodeList = pNode->get_children( "skew" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_SKEW Skew;
		GetSkew( *it, Skew );
		Node.SkewVector.push_back( Skew );
	}

	NodeList = pNode->get_children( "translate" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_TRANSLATE Translate;
		GetTranslate( *it, Translate );
		Node.TranslateVector.push_back( Translate );
	}

	NodeList = pNode->get_children( "boundingbox" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_BOUNDINGBOX BoundingBox;
		GetBoundingBox( *it, BoundingBox );
		Node.BoundingBoxVector.push_back( BoundingBox );
	}

	NodeList = pNode->get_children( "camera" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_CAMERA Camera;
		GetCamera( *it, Camera );
		Node.CameraVector.push_back( Camera );
	}

	NodeList = pNode->get_children( "instance" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_INSTANCE Instance;
		GetInstance( *it, Instance );
		Node.InstanceVector.push_back( Instance );
	}

	// !!!!!!!!!!! Is this reflexive function call safe ?
	// I must check it!!!
	NodeList = pNode->get_children( "node" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_NODE Node;
		GetNode( *it, Node );
		Node.NodeVector.push_back( Node );
	}

	return true;
}

bool COLLADAIO::GetScene( xmlpp::Node *pNode, CLXML_SCENE &Scene )
{
	Scene.name = GetAttributeValue( pNode, "name" );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "lookat" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_LOOKAT LookAt;
		GetLookAt( *it, LookAt );
		Scene.LookAtVector.push_back( LookAt );
	}

	NodeList = pNode->get_children( "matrix" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_MATRIX Matrix;
		GetMatrix( *it, Matrix );
		Scene.MatrixVector.push_back( Matrix );
	}

	NodeList = pNode->get_children( "perspective" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PERSPECTIVE Perspective;
		GetPerspective( *it, Perspective );
		Scene.PerspectiveVector.push_back( Perspective );
	}

	NodeList = pNode->get_children( "rotate" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_ROTATE Rotate;
		GetRotate( *it, Rotate );
		Scene.RotateVector.push_back( Rotate );
	}

	NodeList = pNode->get_children( "scale" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_SCALE Scale;
		GetScale( *it, Scale );
		Scene.ScaleVector.push_back( Scale );
	}

	NodeList = pNode->get_children( "skew" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_SKEW Skew;
		GetSkew( *it, Skew );
		Scene.SkewVector.push_back( Skew );
	}

	NodeList = pNode->get_children( "translate" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_TRANSLATE Translate;
		GetTranslate( *it, Translate );
		Scene.TranslateVector.push_back( Translate );
	}

	NodeList = pNode->get_children( "boundingbox" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_BOUNDINGBOX BoundingBox;
		GetBoundingBox( *it, BoundingBox );
		Scene.BoundingBoxVector.push_back( BoundingBox );
	}

	NodeList = pNode->get_children( "camera" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_CAMERA Camera;
		GetCamera( *it, Camera );
		Scene.CameraVector.push_back( Camera );
	}

	// !!!!!!!!!!! Is this reflexive function call safe ?
	// I must check it!!!
	NodeList = pNode->get_children( "node" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_NODE Node;
		GetNode( *it, Node );
		Scene.NodeVector.push_back( Node );
	}

	return true;
}

bool COLLADAIO::GetGeometry( xmlpp::Node *pNode, CLXML_GEOMETRY &Geometry )
{
	Geometry.name = GetAttributeValue( pNode, "name" );

	xmlpp::Node::NodeList NodeList = pNode->get_children( "mesh" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_MESH Mesh;
		GetMesh( *it, Mesh );
		Geometry.MeshVector.push_back( Mesh );
	}

	NodeList = pNode->get_children( "extra" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_EXTRA Extra;
		GetExtra( *it, Extra );
		Geometry.ExtraVector.push_back( Extra );
	}

	return true;
}

bool COLLADAIO::GetLibrary( xmlpp::Node *pNode, CLXML_LIBRARY &Library )
{
	Library.name = GetAttributeValue( pNode, "name" );
	std::string s = GetAttributeValue( pNode, "type" );
	if( s == "CAMERA" )		Library.type = CLXML_LIBRARY::LIBRARY_CAMERA;
	if( s == "GEOMETRY" )	Library.type = CLXML_LIBRARY::LIBRARY_GEOMETRY;
	if( s == "IMAGE" )		Library.type = CLXML_LIBRARY::LIBRARY_IMAGE;
	if( s == "LIGHT" )		Library.type = CLXML_LIBRARY::LIBRARY_LIGHT;
	if( s == "MATERIAL" )	Library.type = CLXML_LIBRARY::LIBRARY_MATERIAL;
	if( s == "OBJECT" )		Library.type = CLXML_LIBRARY::LIBRARY_OBJECT;
	if( s == "PROGRAM" )	Library.type = CLXML_LIBRARY::LIBRARY_PROGRAM;
	if( s == "TEXTURE" )	Library.type = CLXML_LIBRARY::LIBRARY_TEXTURE;

	xmlpp::Node::NodeList NodeList = pNode->get_children( "camera" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_CAMERA Camera;
		GetCamera( *it, Camera );
		Library.CameraVector.push_back( Camera );
	}

	NodeList = pNode->get_children( "geometry" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_GEOMETRY Geometry;
		GetGeometry( *it, Geometry );
		Library.GeometryVector.push_back( Geometry );
	}

	NodeList = pNode->get_children( "image" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_IMAGE Image;
		GetImage( *it, Image );
		Library.ImageVector.push_back( Image );
	}

	NodeList = pNode->get_children( "light" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_LIGHT Light;
		GetLight( *it, Light );
		Library.LightVector.push_back( Light );
	}

	NodeList = pNode->get_children( "material" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_MATERIAL Material;
		GetMaterial( *it, Material );
		Library.MaterialVector.push_back( Material );
	}

	NodeList = pNode->get_children( "object" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_OBJECT Object;
		GetObject( *it, Object );
		Library.ObjectVector.push_back( Object );
	}

	NodeList = pNode->get_children( "program" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_PROGRAM Program;
		GetProgram( *it, Program );
		Library.ProgramVector.push_back( Program );
	}

	NodeList = pNode->get_children( "texture" );
	for( xmlpp::Node::NodeList::iterator it = NodeList.begin(); it != NodeList.end(); ++it ){
		CLXML_TEXTURE Texture;
		GetTexture( *it, Texture );
		Library.TextureVector.push_back( Texture );
	}

	return true;
}

// Utility
std::string COLLADAIO::GetAttributeValue( xmlpp::Node *pNode, const std::string &AttributeName )
{
	xmlpp::Attribute *pAttribute = ((xmlpp::Element*)pNode)->get_attribute( AttributeName );
	if( pAttribute ) return pAttribute->get_value();

	return std::string();
}

std::string COLLADAIO::GetContent( xmlpp::Node *pNode )
{
	xmlpp::TextNode* pTextNode = ((xmlpp::Element*)pNode)->get_child_text();
	if( pTextNode ) return pTextNode->get_content();

	return std::string();
}

std::vector < std::string > COLLADAIO::TokenizeStrings( std::string Strings )
{
	std::vector < std::string > TokenVector;

	// minimum tokenizer :)
	char *pBuffer = new char[Strings.length() + 10]; // 10 is a margin.
	strcpy( pBuffer, Strings.c_str() );

	char Seps[] = "\x20\t\n\x0a"; // space tab CR LF
	char *pToken = strtok( pBuffer, Seps );

	while( pToken != NULL ){
		TokenVector.push_back( std::string( pToken ) );
		pToken = strtok( NULL, Seps );
	}

	delete [] pBuffer;

	return TokenVector;
}
