/*
	LWCOLLADA, LightWawe plugin for COLLADA.
	Copyright (C) 2004 Yoshikuni Nomura
	If you want contact to me, please send a e-mail to mail@nomuraz.com .

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __COLLADAIO_H__
#define __COLLADAIO_H__

#include <libxml++/libxml++.h>

#include <string>
#include <vector>

// namespace COLLADAIONAMESPACE{ // If you want to use namespace...

#include "COLLADAIOXMLTypes.h"

struct CL_POLYGONS{
	std::vector < std::vector < double > > PositionVector;
	std::vector < std::vector < long > > IndexVector;

	std::vector < std::vector < double > > TexcoordVector;
	std::vector < std::vector < long > > UVIndexVector;
};

struct CL_MESH{
	std::vector < CL_POLYGONS > Polygons;
};

struct CL_GEOMETRY{
	CL_MESH Mesh;
};

class COLLADAIO{
public:
	COLLADAIO();
	~COLLADAIO();
	bool Load( std::string FilaName );
	bool GetGeometry( std::string Name, CL_GEOMETRY &Geometry );

	bool GetGeometry( std::vector < std::string > &NameVector ); // This function is temporary use.


private:
	bool COLLADAIO::GetGeometryPosition( const std::vector < CLXML_POLYGONS >::iterator it_ply, CL_POLYGONS &Polygons );
	bool COLLADAIO::GetGeometryTexcoord( const std::vector < CLXML_POLYGONS >::iterator it_ply, CL_POLYGONS &Polygons );




	// -- Dataflow Elements --
	bool GetAccessor( xmlpp::Node *pNode, CLXML_ACCESSOR &Accessor );
	bool GetArray( xmlpp::Node *pNode, CLXML_ARRAY &Array );
	bool GetInput( xmlpp::Node *pNode, CLXML_INPUT &Input );
	bool GetParam( xmlpp::Node *pNode, CLXML_PARAM &Param );
	bool GetSource( xmlpp::Node *pNode, CLXML_SOURCE &Source );

	// -- Geometry Container Elements --
	bool GetMesh( xmlpp::Node *pNode, CLXML_MESH &Mesh );

	// -- Collation Elements --
	bool GetLines( xmlpp::Node *pNode, CLXML_LINES &Lines );					// This function is not checked.
	bool GetLinestrips( xmlpp::Node *pNode, CLXML_LINESTRIPS &Linestrips );	// This function is not checked.
	bool GetPolygons( xmlpp::Node *pNode, CLXML_POLYGONS &Polygons );
	bool GetTriangles( xmlpp::Node *pNode, CLXML_TRIANGLES &Triangles );		// This function is not checked.
	bool GetTrifans( xmlpp::Node *pNode, CLXML_TRIFANS &Trifans );				// This function is not checked.
	bool GetTristrips( xmlpp::Node *pNode, CLXML_TRISTRIPS &Tristrips );		// This function is not checked.

	// -- Transformational Elements --
	bool GetLookAt( xmlpp::Node *pNode, CLXML_LOOKAT &LookAt );
	bool GetMatrix( xmlpp::Node *pNode, CLXML_MATRIX &Matrix );
	bool GetPerspective( xmlpp::Node *pNode, CLXML_PERSPECTIVE &Perspective );
	bool GetRotate( xmlpp::Node *pNode, CLXML_ROTATE &Rotate );
	bool GetScale( xmlpp::Node *pNode, CLXML_SCALE &Scale );
	bool GetSkew( xmlpp::Node *pNode, CLXML_SKEW &Skew );
	bool GetTranslate( xmlpp::Node *pNode, CLXML_TRANSLATE &Translate );

	// -- Material and Shading Elements --
	bool GetImage( xmlpp::Node *pNode, CLXML_IMAGE &Image );
	bool GetLight( xmlpp::Node *pNode, CLXML_LIGHT &Light );
	bool GetMaterial( xmlpp::Node *pNode, CLXML_MATERIAL &Material );
	bool GetPass( xmlpp::Node *pNode, CLXML_PASS &Pass );
	bool GetShader( xmlpp::Node *pNode, CLXML_SHADER &Shader );
	bool GetTexture( xmlpp::Node *pNode, CLXML_TEXTURE &Texture );

	// -- Procedural Elements --
	bool GetCode( xmlpp::Node *pNode, CLXML_CODE &Code );
	bool GetEntry( xmlpp::Node *pNode, CLXML_ENTRY &Entry );
	bool GetProgram( xmlpp::Node *pNode, CLXML_PROGRAM &Program );

	// -- Object Elements --
	bool GetCamera( xmlpp::Node *pNode, CLXML_CAMERA &Camera );
	bool GetInstance( xmlpp::Node *pNode, CLXML_INSTANCE &Instance );
	bool GetObject( xmlpp::Node *pNode, CLXML_OBJECT &Object ); // This function is not checked.
	bool GetAsset( xmlpp::Node *pNode, CLXML_ASSET &Asset );
	bool GetExtra( xmlpp::Node *pNode, CLXML_EXTRA &Extra ); // This function is not implemented completely.

	// -- Hierarchical Elements --
	bool GetBoundingBox( xmlpp::Node *pNode, CLXML_BOUNDINGBOX &BoundingBox );
	bool GetNode( xmlpp::Node *pNode, CLXML_NODE &Node );
	bool GetScene( xmlpp::Node *pNode, CLXML_SCENE &Scene );
	
	// -- Categorical Elements --
	bool GetGeometry( xmlpp::Node *pNode, CLXML_GEOMETRY &Geometry );
	bool GetLibrary( xmlpp::Node *pNode, CLXML_LIBRARY &Library );

	// Utility
	std::string GetAttributeValue( xmlpp::Node *pNode, const std::string &AttributeName );
	std::string GetContent( xmlpp::Node *pNode );
	std::vector < std::string > TokenizeStrings( std::string Strings );

private:
	xmlpp::DomParser *pPaser;
	xmlpp::Document *pDocument;
};

// }; // If you want to use namespace...


#endif // __COLLADAIO_H__
