/*
	LWCOLLADA, LightWawe plugin for COLLADA.
	Copyright (C) 2004 Yoshikuni Nomura
	If you want contact to me, please send a e-mail to mail@nomuraz.com .

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __COLLADAIOXMLTYPES_H__
#define __COLLADAIOXMLTYPES_H__

typedef std::vector< double > CTXML_LISTOFFLOATS;
typedef std::vector< long > CTXML_LISTOFINTS;
typedef std::vector< std::string > CTXML_LISTOFTOKENS;
typedef CTXML_LISTOFFLOATS CTXML_COLORRGB;
typedef CTXML_LISTOFFLOATS CTXML_COLORRGBA;
typedef CTXML_LISTOFFLOATS CTXML_MATRIX4;
typedef CTXML_LISTOFFLOATS CTXML_FLOAT2;
typedef CTXML_LISTOFFLOATS CTXML_FLOAT3;
typedef CTXML_LISTOFFLOATS CTXML_FLOAT4;
typedef CTXML_LISTOFINTS CTXML_INT2;
typedef CTXML_LISTOFINTS CTXML_INT3;
typedef CTXML_LISTOFINTS CTXML_INT4;

struct CLXML_UNIT{
	// attribute
	double meter;		// xs:double
	std::string name;	// xs:NMTOKEN

	CLXML_UNIT(){ meter = 1.000; name = "meter"; } // default
};

struct CLXML_ASSET{
	enum UPAXISTYPE{
		UPAXIS_X_UP = 0,
		UPAXIS_Y_UP,
		UPAXIS_Z_UP,

		UPAXIS_UNINITIALIZED
	};

	// elemant
	std::string author;			// xs:string
	std::string authoring_tool;	// xs:string
	std::string created;		// xs:dateTime
	std::string modified;		// xs:dateTime
	std::string revision;		// xs:string
	std::string source_data;	// xs:anyURI
	std::string copyright;		// xs:string
	std::string title;			// xs:string
	std::string subject;		// xs:string
	std::string keywords;		// xs:string
	std::string comments;		// xs:string
	UPAXISTYPE up_axis;			// UpAxisType

	// element
	std::vector < CLXML_UNIT > UnitVector; // maxOccurs="1"

	CLXML_ASSET(){ up_axis = UPAXIS_UNINITIALIZED; };
};

struct CLXML_ARRAY{
	enum ARRAYTYPES{
		ARRAY_FLOAT = 0,
		ARRAY_INT,
		ARRAY_NAME,
		ARRAY_TOKEN
	};

	// attribute
	unsigned long count;	// xs:nonNegativeInteger
	std::string	name;		// xs:ID
	ARRAYTYPES type;		// ArrayTypes

	// contents
	std::vector < std::string > ContentTokenVector;
};

struct CLXML_INPUT{
	// attribute
	std::string	id;			// xs:NMTOKEN
	std::string	semantic;	// xs:NMTOKEN
	std::string	source;		// xs:anyURI
};

struct CLXML_PARAM{
	enum FLOWTYPE{
		FLOW_IN = 0,
		FLOW_OUT,
		FLOW_INOUT,

		FLOW_UNINITIALIZED
	};

	// attribute
	FLOWTYPE flow;			// FlowType
	std::string	name;		// xs:NCName
	std::string	semantic;	// xs:token
	std::string	type;		// xs:NMTOKEN

	// This is only contained in 'entry node'.
	std::string qualifier;;	// xs:NMTOKEN

	// contents
	std::vector < std::string > ContentTokenVector;

	CLXML_PARAM(){ flow = FLOW_UNINITIALIZED; }
};

struct CLXML_ACCESSOR{
	// attribute
	unsigned long count;	// xs:nonNegativeInteger
	unsigned long offset;	// xs:nonNegativeInteger
	std::string	source;		// xs:anyURI
	unsigned long stride;	// xs:nonNegativeInteger

	// element
	std::vector < CLXML_PARAM > ParamVector;

	CLXML_ACCESSOR(){ offset = 0; stride = 1; } // default
};

struct CLXML_ENTRY{
	// attribute
	std::string	function; // xs:NMTOKEN
	std::string	semantic; // xs:NMTOKEN

	// element
	std::vector < CLXML_PARAM > ParamVector;
};

struct CLXML_CODE{
	// attribute
	std::string	profile;	// xs:string
	std::string	semantic;	// xs:NMTOKEN
	std::string	url;		// xs:anyURI

	// contents
	std::string Content;
};

struct CLXML_PROGRAM{
	// attribute
	std::string	name;	// xs:Name
	std::string	url;	// xs:anyURI

	// element
	std::vector < CLXML_ASSET > AssetVector; // minOccurs="0" maxOccurs="1"
	std::vector < CLXML_PARAM > ParamVector;
	std::vector < CLXML_ENTRY > EntryVector;
	std::vector < CLXML_CODE > CodeVector;
};

struct CLXML_IMAGE{
	// attribute
	std::string	name;		// xs:Name
	std::string	source;		// xs:anyURI
	std::string	format;		// xs:string
	unsigned long height;	// xs:nonNegativeInteger
	unsigned long width;	// xs:nonNegativeInteger
	unsigned long depth;	// xs:nonNegativeInteger

	CLXML_IMAGE(){ depth = 1; };
};

struct CLXML_LIGHT{
	enum LIGHTTYPE{
		LIGHT_AMBIENT = 0,
		LIGHT_DIRECTIONAL,
		LIGHT_POINT,
		LIGHT_SPOT
	};

	// attribute
	std::string	name;	// xs:ID
	LIGHTTYPE type;		// LightType

	// element
	std::vector < CLXML_ASSET > AssetVector; // minOccurs="0" maxOccurs="1"
	std::vector < CLXML_PARAM > ParamVector;

	CLXML_LIGHT(){ type = LIGHT_POINT; }
};

struct CLXML_PASS{
	// element
	std::vector < CLXML_PARAM >   ParamVector;
	std::vector < CLXML_INPUT >   InputVector;
	std::vector < CLXML_PROGRAM > ProgramVector;
};

struct CLXML_TECHNIQUE_SOURCE{
	// attribute
	std::string	profile; // xs:string

	// element
	std::vector < CLXML_ASSET > AssetVector; // minOccurs="0" maxOccurs="1"
	std::vector < CLXML_ACCESSOR > AccessorVector;
	std::vector < CLXML_PARAM > ParamVector;
	std::vector < CLXML_PROGRAM > ProgramVector; // maxOccurs="1"
};

struct CLXML_SOURCE{
	// attribute
	std::string	name; // xs:ID

	// element
	std::vector < CLXML_ARRAY > ArrayVector; // maxOccurs="1"
	std::vector < CLXML_TECHNIQUE_SOURCE > TechniqueVector;
};

struct CLXML_TECHNIQUE_SHADER{
	// attribute
	std::string	profile; // xs:string

	// element
	std::vector < CLXML_ASSET > AssetVector; // minOccurs="0" maxOccurs="1"
	std::vector < CLXML_PARAM > ParamVector;
	std::vector < CLXML_PASS > PassVector;
	std::vector < CLXML_PROGRAM > ProgramVector; // maxOccurs="1"
};

struct CLXML_SHADER{
	// attribute
	std::string	name; // xs:NCName

	// element
	std::vector < CLXML_PARAM > ParamVector;
	std::vector < CLXML_TECHNIQUE_SHADER > TechniqueVector;
};

struct CLXML_TECHNIQUE_TEXTURE{
	// attribute
	std::string	profile; // xs:string

	// element
	std::vector < CLXML_ASSET > AssetVector; // minOccurs="0" maxOccurs="1"
	std::vector < CLXML_INPUT > InputVector;
	std::vector < CLXML_PARAM > ParamVector;
	std::vector < CLXML_PROGRAM > ProgramVector; // maxOccurs="1"
};

struct CLXML_TEXTURE{
	// attribute
	std::string	name; // xs:ID

	// element
	std::vector < CLXML_ASSET > AssetVector; // minOccurs="0" maxOccurs="1"
	std::vector < CLXML_PARAM > ParamVector;
	std::vector < CLXML_TECHNIQUE_TEXTURE > TechniqueVector;
};

struct CLXML_MATERIAL{
	// attribute
	std::string	name; // xs:ID

	// element
	std::vector < CLXML_ASSET > AssetVector; // minOccurs="0" maxOccurs="1"
	std::vector < CLXML_PARAM > ParamVector;
	std::vector < CLXML_SHADER > ShaderVector;
};

struct CLXML_LINES{
	// attribute
	unsigned long count;	// xs:nonNegativeInteger
	std::string material;	// xs:anyURI

	// element
	std::vector < CLXML_INPUT > InputVector;
	std::vector < CLXML_PARAM > ParamVector;
	std::vector< CTXML_LISTOFINTS > PVector;
};

struct CLXML_LINESTRIPS{
	// attribute
	unsigned long count;	// xs:nonNegativeInteger
	std::string material;	// xs:anyURI

	// element
	std::vector < CLXML_INPUT > InputVector;
	std::vector < CLXML_PARAM > ParamVector;
	std::vector< CTXML_LISTOFINTS > PVector;
};

struct CLXML_P{
	CTXML_LISTOFINTS p;
	CTXML_LISTOFINTS h;
};

struct CLXML_POLYGONS{
	// attribute
	unsigned long count;	// xs:nonNegativeInteger
	std::string material;	// xs:anyURI

	// element
	std::vector < CLXML_INPUT > InputVector;
	std::vector < CLXML_PARAM > ParamVector;
	std::vector < CLXML_P > PVector;
};

struct CLXML_TRIANGLES{
	// attribute
	unsigned long count;	// xs:nonNegativeInteger
	std::string material;	// xs:anyURI

	// element
	std::vector < CLXML_INPUT > InputVector;
	std::vector < CLXML_PARAM > ParamVector;
	std::vector< CTXML_LISTOFINTS > PVector;
};

struct CLXML_TRIFANS{
	// attribute
	unsigned long count;	// xs:nonNegativeInteger
	std::string material;	// xs:anyURI

	// element
	std::vector < CLXML_INPUT > InputVector;
	std::vector < CLXML_PARAM > ParamVector;
	std::vector< CTXML_LISTOFINTS > PVector;
};

struct CLXML_TRISTRIPS{
	// attribute
	unsigned long count;	// xs:nonNegativeInteger
	std::string material;	// xs:anyURI

	// element
	std::vector < CLXML_INPUT > InputVector;
	std::vector < CLXML_PARAM > ParamVector;
	std::vector< CTXML_LISTOFINTS > PVector;
};

struct CLXML_MESH{
	// attribute
	std::string name; // xs:NCName

	// element
	std::vector < CLXML_SOURCE > SourceVector;
	std::vector < CLXML_LINES > LinesVector;
	std::vector < CLXML_LINESTRIPS > LinestripsVector;
	std::vector < CLXML_POLYGONS > PolygonsVector;
	std::vector < CLXML_TRIANGLES > TrianglesVector;
	std::vector < CLXML_TRIFANS > TrifansVector;
	std::vector < CLXML_TRISTRIPS > TristripsVector;
};

struct CLXML_LOOKAT{
	// contents
	CTXML_LISTOFFLOATS ListOfFloats;
};

struct CLXML_MATRIX{
	// contents
	CTXML_MATRIX4 Matrix4;
};

struct CLXML_PERSPECTIVE{
	// contents
	double Float;
};

struct CLXML_ROTATE{
	// contents
	CTXML_FLOAT4 Float4;
};

struct CLXML_SCALE{
	// contents
	CTXML_FLOAT3 Float3;
};

struct CLXML_SKEW{
	// contents
	CTXML_LISTOFFLOATS ListOfFloats;
};

struct CLXML_TRANSLATE{
	// contents
	CTXML_FLOAT3 Float3;
};

struct CLXML_OPTICS{
	std::vector < CLXML_PROGRAM > ProgramVector;
};

struct CLXML_IMAGER{
	std::vector < CLXML_PROGRAM > ProgramVector;
};

struct CLXML_CAMERA_TECHNIQUE{
	// attribute
	std::string profile; // xs:string

	// element
	std::vector < CLXML_OPTICS > OpticsVector;
	std::vector < CLXML_IMAGER > ImagerVector;
};

struct CLXML_CAMERA{
	// attribute
	std::string name; // xs:ID

	// element
	std::vector < CLXML_ASSET > AssetVector; // minOccurs="0" maxOccurs="1"
	std::vector < CLXML_CAMERA_TECHNIQUE > TechniqueVector;
};

struct CLXML_INSTANCE{
	// attribute
	std::string url; // xs:anyURI
};

struct CLXML_OBJECTXML_TECHNIQUE{
	// attribute
	std::string profile; // xs:string

	// element
	std::vector < CLXML_INPUT > InputVector;
	std::vector < CLXML_PROGRAM > ProgramVector;
};

struct CLXML_MODIFIER{
	std::vector < CLXML_OBJECTXML_TECHNIQUE > TechniqueVector;
};

struct CLXML_OBJECT{
	// attribute
	std::string name; // xs:ID

	// element
	std::vector < CLXML_MODIFIER > ModifierVector;
};

struct CLXML_EXTRA{
	// attribute
	std::string name; // xs:NCName
};

struct CLXML_BOUNDINGBOX{
	// contents
	CTXML_LISTOFFLOATS Min;
	CTXML_LISTOFFLOATS Max;
};

struct CLXML_NODE{
	// attribute
	std::string name; // xs:NCName

	// element
	std::vector < CLXML_LOOKAT > LookAtVector;
	std::vector < CLXML_MATRIX > MatrixVector;
	std::vector < CLXML_PERSPECTIVE > PerspectiveVector;
	std::vector < CLXML_ROTATE > RotateVector;
	std::vector < CLXML_SCALE > ScaleVector;
	std::vector < CLXML_SKEW > SkewVector;
	std::vector < CLXML_TRANSLATE > TranslateVector;
	std::vector < CLXML_BOUNDINGBOX > BoundingBoxVector; // maxOccurs="1"
	std::vector < CLXML_CAMERA > CameraVector;
	std::vector < CLXML_INSTANCE > InstanceVector;
	std::vector < CLXML_NODE > NodeVector;
};

struct CLXML_SCENE{
	// attribute
	std::string name; // xs:NCName

	// element
	std::vector < CLXML_LOOKAT > LookAtVector;
	std::vector < CLXML_MATRIX > MatrixVector;
	std::vector < CLXML_PERSPECTIVE > PerspectiveVector;
	std::vector < CLXML_ROTATE > RotateVector;
	std::vector < CLXML_SCALE > ScaleVector;
	std::vector < CLXML_SKEW > SkewVector;
	std::vector < CLXML_TRANSLATE > TranslateVector;
	std::vector < CLXML_BOUNDINGBOX > BoundingBoxVector; // maxOccurs="1"
	std::vector < CLXML_CAMERA > CameraVector;
	std::vector < CLXML_NODE > NodeVector;
};

struct CLXML_GEOMETRY{
	// attribute
	std::string name; // xs:ID

	// element
	std::vector < CLXML_MESH > MeshVector;
	std::vector < CLXML_EXTRA > ExtraVector;
};

struct CLXML_LIBRARY{
	enum LIBRARYTYPE{
		LIBRARY_CAMERA,
		LIBRARY_GEOMETRY,
		LIBRARY_IMAGE,
		LIBRARY_LIGHT,
		LIBRARY_MATERIAL,
		LIBRARY_OBJECT,
		LIBRARY_PROGRAM,
		LIBRARY_TEXTURE,

		LIBRARY_UNINITIALIZED
	};


	// attribute
	std::string name; // xs:NCName
	LIBRARYTYPE type; // LibraryType

	// element
	std::vector < CLXML_CAMERA > CameraVector;
	std::vector < CLXML_GEOMETRY > GeometryVector;
	std::vector < CLXML_IMAGE > ImageVector;
	std::vector < CLXML_LIGHT > LightVector;
	std::vector < CLXML_MATERIAL > MaterialVector;
	std::vector < CLXML_OBJECT > ObjectVector;
	std::vector < CLXML_PROGRAM > ProgramVector;
	std::vector < CLXML_TEXTURE > TextureVector;

	CLXML_LIBRARY(){ type = LIBRARY_UNINITIALIZED;};
};

#endif // __COLLADAIOXMLTYPES_H__
