/*
	LWCOLLADA, LightWawe plugin for COLLADA.
	Copyright (C) 2004 Yoshikuni Nomura
	If you want contact to me, please send a e-mail to mail@nomuraz.com .

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <lwserver.h>
#include <lwobjimp.h>

#include "LWSurfaceChunk.h"

#include "COLLADAIO.h"

XCALL_( int ) LWCOLLADALoad( long Version, GlobalFunc *pGlobal, LWObjectImport *pImport, void *pServerData )
{
	if ( Version != LWOBJECTIMPORT_VERSION ) return AFUNC_BADVERSION;

	COLLADAIO CLIO;
	CLIO.Load( pImport->filename );

	std::vector < std::string > NameVector;
	CLIO.GetGeometry( NameVector );

	for( unsigned int l = 0; l < NameVector.size(); ++l ){
		CL_GEOMETRY Geometry;
		CLIO.GetGeometry( NameVector[l].c_str(), Geometry );

		pImport->layer( pImport->data, l, NULL );

		LWFVector Pos = { 0, 0, 0 };
		pImport->pivot( pImport->data, Pos );

		// Create UV coordinates
		pImport->vmap( pImport->data, LWVMAP_TXUV, 2, ( std::string( "uv_" ) + NameVector[l] ).c_str() );

		LWSurfaceChunk::SURFACECHUNKDATA Data;
		Data.Color = LWSurfaceChunk::COL12( 0.5f, 0.5f, 0.5f );
		Data.UVMapName = ( std::string( "uv_" ) + NameVector[l] ).c_str();
		LWSurfaceChunk Chunk;
		Chunk.Create( Data );

		pImport->surface( pImport->data, ( std::string( "surf_" ) + NameVector[l] ).c_str(), NULL, Chunk.GetSize(), Chunk.GetChunkPointer() );

		for( unsigned int k = 0; k < Geometry.Mesh.Polygons.size(); ++k ){
			CL_POLYGONS &Polygons = Geometry.Mesh.Polygons[k];

			// Create positions
			unsigned int PositionVectorSize = (unsigned int)Polygons.PositionVector.size();
			LWPntID *pPosition = new LWPntID[Polygons.PositionVector.size()];
			for( unsigned int j = 0; j < Polygons.PositionVector.size(); ++j ){
				unsigned int Dimention = (unsigned int)Polygons.PositionVector[j].size();
				LWFVector Pos = { 0, 0, 0 };
				for( unsigned int i = 0; i < Dimention; ++i ) Pos[i] = (float)Polygons.PositionVector[j][i];

				if( !( pPosition[j] = pImport->point( pImport->data, Pos ) ) ){
					goto Finish;
				}
			}

			// Create polygons
			for( unsigned int k = 0; k < Polygons.IndexVector.size(); ++k ){
				unsigned int Dimention = (unsigned int)Polygons.IndexVector[k].size();
				LWPntID *pIndex = new LWPntID[Dimention];
				for( unsigned int j = 0; j < Dimention; ++j ) pIndex[j] = pPosition[Polygons.IndexVector[k][j]];

				LWPolID polID = pImport->polygon( pImport->data, LWPOLTYPE_FACE, 0, Dimention, pIndex );

				pImport->polTag( pImport->data, polID, LWPTAG_SURF, ( std::string( "surf_" ) + NameVector[l] ).c_str() );

				for( unsigned int j = 0; j < Dimention; ++j ){
					// !!!!! CAUTION
					// UV Index in COLLADA file seems to support plural UV value for one vertex, but LW can't. ????
					if( Polygons.UVIndexVector.size() > k ){ // Polygons have UV or not?
						if( Polygons.UVIndexVector[k].size() > j ){
							float uv[2];
							for( int i = 0; i < 2; ++i ) uv[i] = (float)Polygons.TexcoordVector[Polygons.UVIndexVector[k][j]][i];

							pImport->vmapVal( pImport->data, pIndex[j], uv );
						}
					}
				}

				delete [] pIndex;
			}

Finish:
			delete [] pPosition;
		}
	}

	pImport->done( pImport->data );

	return AFUNC_OK;
}
