/*
	LWCOLLADA, LightWawe plugin for COLLADA.
	Copyright (C) 2004 Yoshikuni Nomura
	If you want contact to me, please send a e-mail to mail@nomuraz.com .

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "LWChunk.h"

LWChunk::LWChunk()
: pChunk( NULL )
{
}

void LWChunk::SetChunk( std::vector < unsigned char > Chunk )
{
	for( unsigned int i = 0; i < Chunk.size(); ++i ) this->Chunk.push_back( Chunk[i] );
}

LWChunk::~LWChunk()
{
	if( pChunk ) delete [] pChunk;
}

void LWChunk::SetInt1( const char Data )
{
	unsigned char *p = (unsigned char*)&Data;
	Chunk.push_back( *p );
}

void LWChunk::SetInt2( const short Data )
{
	unsigned char *p = (unsigned char*)&Data;
	for( int i = 1; i >= 0; --i ) Chunk.push_back( p[i] );
}

void LWChunk::SetInt4( const long Data )
{
	unsigned char *p = (unsigned char*)&Data;
	for( int i = 3; i >= 0; --i ) Chunk.push_back( p[i] );
}

void LWChunk::SetUInt1( const unsigned char Data )
{
	Chunk.push_back( Data );
}

void LWChunk::SetUInt2( const unsigned short Data )
{
	unsigned char *p = (unsigned char*)&Data;
	for( int i = 1; i >= 0; --i ) Chunk.push_back( p[i] );
}

void LWChunk::SetUInt4( const unsigned long Data )
{
	unsigned char *p = (unsigned char*)&Data;
	for( int i = 3; i >= 0; --i ) Chunk.push_back( p[i] );
}

void LWChunk::SetFloat( const float Data )
{
	unsigned char *p = (unsigned char*)&Data;
	for( int i = 3; i >= 0; --i ) Chunk.push_back( p[i] );
}

void LWChunk::SetString( const std::string String )
{
	for( unsigned int i = 0; i < String.length(); ++i ) Chunk.push_back( String.c_str()[i] );
	Chunk.push_back( 0 ); // '\0'
}

unsigned char* LWChunk::GetChunkPointer()
{
	if( pChunk ) delete [] pChunk;
	pChunk = new unsigned char[Chunk.size() + 10]; // +10 is for safety
	for( unsigned int i = 0; i < Chunk.size(); ++i ) pChunk[i] = Chunk[i];

	return pChunk;
}

unsigned int LWChunk::GetSize()
{
	return (unsigned int)Chunk.size();
}
