/*
	LWCOLLADA, LightWawe plugin for COLLADA.
	Copyright (C) 2004 Yoshikuni Nomura
	If you want contact to me, please send a e-mail to mail@nomuraz.com .

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "LWSurfaceChunk.h"

void LWSurfaceChunk::Create( SURFACECHUNKDATA Data )
{
	SetString( "COLR" );
	SetInt1( 0x0e );
	SetFloat( Data.Color.r );
	SetFloat( Data.Color.g );
	SetFloat( Data.Color.b );
	SetInt1( 0 );
	SetInt1( 0 );

	LWChunk Channel;
	Channel.SetString( "CHAN" );
	Channel.SetInt1( 4 );
	Channel.SetInt1( 'C' );
	Channel.SetInt1( 'O' );
	Channel.SetInt1( 'L' );
	Channel.SetInt1( 'R' );

	LWChunk Opacity;
	Opacity.SetString( "OPAC" );
	Opacity.SetInt1( 8 );
	Opacity.SetInt2( 0 );
	Opacity.SetFloat( 1 );
	Opacity.SetInt2( 0 );

	LWChunk Enable;
	Enable.SetString( "ENAB" );
	Enable.SetInt1( 2 );
	Enable.SetInt2( 1 );

	LWChunk Negative;
	Negative.SetString( "NEGA" );
	Negative.SetInt1( 2 );
	Negative.SetInt2( 0 );

	LWChunk ImageMap;
	ImageMap.SetString( "IMAP" );
	ImageMap.SetInt1( 42 ); // !!!!!!!
	ImageMap.SetUInt1( 0x80 );// !!!!!!!
	ImageMap.SetInt1( 0 );// !!!!!!!

	ImageMap.SetChunk( Channel.GetChunk() );
	ImageMap.SetChunk( Opacity.GetChunk() );
	ImageMap.SetChunk( Enable.GetChunk() );
	ImageMap.SetChunk( Negative.GetChunk() );

	LWChunk Center;
	Center.SetString( "CNTR" );
	Center.SetInt1( 14 );
	Center.SetFloat( 0 );
	Center.SetFloat( 0 );
	Center.SetFloat( 0 );
	Center.SetInt2( 0 );

	LWChunk Size;
	Size.SetString( "SIZE" );
	Size.SetInt1( 14 );
//	Size.SetFloat( 1 );
//	Size.SetFloat( 1 );
//	Size.SetFloat( 1 );

Size.SetFloat( 1 );
Size.SetFloat( 2 );
Size.SetFloat( 3 );

	Size.SetInt2( 0 );

	LWChunk Rotate;
	Rotate.SetString( "ROTA" );
	Rotate.SetInt1( 14 );
	Rotate.SetFloat( 0 );
	Rotate.SetFloat( 0 );
	Rotate.SetFloat( 0 );
	Rotate.SetInt2( 0 );

	LWChunk Falloff;
	Falloff.SetString( "FALL" );
	Falloff.SetInt1( 16 );
	Falloff.SetInt2( 0 );
	Falloff.SetFloat( 0 );
	Falloff.SetFloat( 0 );
	Falloff.SetFloat( 0 );
	Falloff.SetInt2( 0 );

	LWChunk ObjectRefer;
	ObjectRefer.SetString( "OREF" );
	ObjectRefer.SetInt1( 2 ); // !!!!!!
	ObjectRefer.SetInt1( 0 ); // !!!!!!
	ObjectRefer.SetInt1( 0 ); // !!!!!!

	LWChunk CoordinateSystem;
	CoordinateSystem.SetString( "CSYS" );
	CoordinateSystem.SetInt1( 2 );
	CoordinateSystem.SetInt2( 0 );

	LWChunk TextureMap;
	TextureMap.SetString( "TMAP" );
	TextureMap.SetInt1( 98 ); // !!!!!!!

	TextureMap.SetChunk( Center.GetChunk() );
	TextureMap.SetChunk( Size.GetChunk() );
	TextureMap.SetChunk( Rotate.GetChunk() );
	TextureMap.SetChunk( Falloff.GetChunk() );
	TextureMap.SetChunk( ObjectRefer.GetChunk() );
	TextureMap.SetChunk( CoordinateSystem.GetChunk() );

	LWChunk Projection;
	Projection.SetString( "PROJ" );
	Projection.SetInt1( 2 );
	Projection.SetInt2( 5 );

	LWChunk Axis;
	Axis.SetString( "AXIS" );
	Axis.SetInt1( 2 );
	Axis.SetInt2( 2 );

	LWChunk Image;
	Image.SetString( "IMAG" );
	Image.SetInt1( 2 );
	Image.SetInt2( 1 );

	LWChunk Wrap;
	Wrap.SetString( "WRAP" );
	Wrap.SetInt1( 4 );
	Wrap.SetInt2( 1 );
	Wrap.SetInt2( 1 );

	LWChunk Wrpw;
	Wrpw.SetString( "WRPW" );
	Wrpw.SetInt1( 6 );
	Wrpw.SetFloat( 1 );
	Wrpw.SetInt2( 0 );

	LWChunk Wrph;
	Wrph.SetString( "WRPH" );
	Wrph.SetInt1( 6 );
	Wrph.SetFloat( 1 );
	Wrph.SetInt2( 0 );

	LWChunk Vmap;
	Vmap.SetString( "VMAP" );

	int Padding = 0;
	if( Data.UVMapName.size() % 2 == 0 ) Padding = 1;
	Vmap.SetInt1( (unsigned int )Data.UVMapName.size() + 1 + Padding ); // +1 means '\0'
	Vmap.SetString( Data.UVMapName );
	if( Padding == 1 ) Vmap.SetInt1( 0 );

	LWChunk Aast;
	Aast.SetString( "AAST" );
	Aast.SetInt1( 6 );
	Aast.SetInt2( 1 );
	Aast.SetFloat( 1 );

	LWChunk Pixb;
	Pixb.SetString( "PIXB" );
	Pixb.SetInt1( 2 );
	Pixb.SetInt2( 1 );

	LWChunk Block;
	Block.SetString( "BLOK" );

	int ChunkSize = ImageMap.GetSize() + TextureMap.GetSize() + Projection.GetSize() + Axis.GetSize() + Image.GetSize()
	+ Wrap.GetSize() + Wrpw.GetSize() + Wrph.GetSize() + Vmap.GetSize()/* + Aast.GetSize() + Pixb.GetSize()*/;

	Block.SetUInt1( ChunkSize );
	Block.SetChunk( ImageMap.GetChunk() );
	Block.SetChunk( TextureMap.GetChunk() );

	Block.SetChunk( Projection.GetChunk() );
	Block.SetChunk( Axis.GetChunk() );
	Block.SetChunk( Image.GetChunk() );
	Block.SetChunk( Wrap.GetChunk() );
	Block.SetChunk( Wrpw.GetChunk() );
	Block.SetChunk( Wrph.GetChunk() );
	Block.SetChunk( Vmap.GetChunk() );
//	Block.SetChunk( Aast.GetChunk() );
//	Block.SetChunk( Pixb.GetChunk() );

	FILE *fp = fopen( "zzz.txt", "wt" );
	int a = Block.GetSize();
	for( int i = 0; i < a; ++i ){
		fprintf( fp, "%02x ", *( Block.GetChunkPointer() + i ) );
	}
	fclose(fp);

	SetChunk( Block.GetChunk() );
}

